unit superobjecthelper;

{$I 'qdac.inc'}

interface
uses qstring,qjson,qrbtree;
type

  TSuperObjectHelper=class(TQJson)
  private
    function GetS(APath: QStringW): QStringW;
    procedure SetS(APath: QStringW; const Value: QStringW);
    function GetB(APath: QStringW): Boolean;
    function GetD(APath: QStringW): TDateTime;
    function GetF(APath: QStringW): Extended;
    function GetI(APath: QStringW): Int64;
    procedure SetB(APath: QStringW; const Value: Boolean);
    procedure SetD(APath: QStringW; const Value: TDateTime);
    procedure SetF(APath: QStringW; const Value: Extended);
    procedure SetI(APath: QStringW; const Value: Int64);
  protected
    FHashTable:TQHashTable;
  public
    constructor Create;overload;
    destructor Destroy;override;
    function Add(AName:QStringW):TQJson;override;
    function IndexOf(const AName:QStringW):Integer;override;
    procedure Delete(AIndex:Integer);override;
    procedure Clear;override;
    property S[APath:QStringW]:QStringW read GetS write SetS;
    property B[APath:QStringW]:Boolean read GetB write SetB;
    property I[APath:QStringW]:Int64 read GetI write SetI;
    property F[APath:QStringW]:Extended read GetF write SetF;
    property D[APath:QStringW]:TDateTime read GetD write SetD;
  end;
implementation
uses dialogs,sysutils;
{ TSuperObjectHelper }

function TSuperObjectHelper.Add(AName: QStringW): TQJson;
begin
Result:=inherited Add(AName);
//classes
FNameHash:=HashOf(PQCharW(AName),Length(AName) shl 1);
FHashTable.Add(Pointer(Count-1),FNameHash);
end;

procedure TSuperObjectHelper.Clear;
begin
  inherited;
FHashTable.Clear;
end;

constructor TSuperObjectHelper.Create;
begin
FHashTable:=TQHashTable.Create();
FHashTable.AutoSize:=True;
end;

procedure TSuperObjectHelper.Delete(AIndex: Integer);
var
  AItem:TQJson;
begin
AItem:=Items[AIndex];
FHashTable.Delete(Pointer(AIndex),AItem.NameHash);
inherited;
end;

destructor TSuperObjectHelper.Destroy;
begin
FreeObject(FHashTable);
  inherited;
end;

function TSuperObjectHelper.GetB(APath: QStringW): Boolean;
begin
Result:=ForcePath(APath).AsBoolean;
end;

function TSuperObjectHelper.GetD(APath: QStringW): TDateTime;
begin
Result:=ForcePath(APath).AsDateTime;
end;

function TSuperObjectHelper.GetF(APath: QStringW): Extended;
begin
Result:=ForcePath(APath).AsFloat;
end;

function TSuperObjectHelper.GetI(APath: QStringW): Int64;
begin
Result:=ForcePath(APath).AsInteger;
end;

function TSuperObjectHelper.GetS(APath: QStringW): QStringW;
begin
Result:=ValueByPath(APath,'');
end;

function TSuperObjectHelper.IndexOf(const AName: QStringW): Integer;
var
  AIndex,AHash:Integer;
  AList:PQHashList;
  AItem:TQJson;
begin
AHash:=HashOf(PQCharW(AName),Length(AName) shl 1);
AList:=FHashTable.FindFirst(AHash);
Result:=-1;
while AList<>nil do
  begin
  AIndex:=Integer(AList.Data);
  AItem:=Items[AIndex];
  if AItem.Name=AName then
    begin
    Result:=AIndex;
    Break;
    end
  else
    AList:=FHashTable.FindNext(AList);
  end;
end;

procedure TSuperObjectHelper.SetB(APath: QStringW; const Value: Boolean);
begin
ForcePath(APath).AsBoolean:=Value;
end;

procedure TSuperObjectHelper.SetD(APath: QStringW; const Value: TDateTime);
begin
ForcePath(APath).AsDateTime:=Value;
end;

procedure TSuperObjectHelper.SetF(APath: QStringW; const Value: Extended);
begin
ForcePath(APath).AsFloat:=Value;
end;

procedure TSuperObjectHelper.SetI(APath: QStringW; const Value: Int64);
begin
ForcePath(APath).AsInt64:=Value;
end;

procedure TSuperObjectHelper.SetS(APath: QStringW; const Value: QStringW);
begin
ForcePath(APath).AsString:=Value;
end;

end.
